game = class:new()

function game:init()
	self.moves = 0
	self.grid = {}
	if loaded then
		used = 0	-- will increase in grid-making loop
	end
	local data = love.image.newImageData(16,16)	-- PO2 image, only 15x15 used
	for y=1, 15 do	-- loop through the rows first
		self.grid[y] = {}
		for x=1, 15 do	-- now do the columns
			local c
			if loaded then
				c = loaded[y][x]
				if c > used then
					used = c
				end
			else
				c = math.random(1, used)
			end
			self.grid[y][x] = c
			data:setPixel(x-1, y-1, colors[c][1], colors[c][2], colors[c][3], 255)
		end
	end
	loaded = nil		-- unset so it doesn't get used again
	self.img = love.graphics.newImage(data)		-- the original colors are saved into an image for easy rendering
	self.img:setFilter("nearest", "nearest")
	-- set up first move
	self.current = self.grid[1][1]
	self.cursor = self.current
	self:dye(1, 1)
end

function game:update(dt)
	local mX, mY = love.mouse.getPosition()
	if mX ~= mouseX or mY ~= mouseY then	-- only do when we move
		if mY > 32 and mY < (used * 48) + 8 then	-- only do when we're in range
			self.cursor = math.min(math.max(math.floor((mY + 8) / 48), 1), used)	-- find the cursor equivalent, and keep it in bounds
		end
		mouseX, mouseY = mX, mY
	end
end

function game:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(self.img, 0, 0, 0, 32, 32)		-- the original board of colors
	love.graphics.print("Move "..self.moves, 512, 8)
	love.graphics.setColor(colors[self.current])
	for y=1, 15 do		-- let's render the squares we've dyed
		for x=1, 15 do	-- remember, rows (y) then columns (x)
			if self.grid[y][x] == 0 then
				love.graphics.rectangle("fill", x * 32 - 32, y * 32 - 32, 32, 32)
			end
		end
	end
	for i=1, used do		-- lets render the available colors
		love.graphics.setColor(colors[i])
		love.graphics.rectangle("fill", 512, i * 48, 32, 32)
	end
	love.graphics.setColor(0,0,0)
	for i=1, used do		-- ...and overlay numbers on top of ease of use
		love.graphics.print(i, 524, i * 48)
	end
	love.graphics.setColor(colors[self.cursor])
	love.graphics.rectangle("fill", 560, (self.cursor * 48) + 8, 16, 16)
end

function game:keypressed(key, unicode)
	if key == "escape" or key == "backspace" then
		sfx:play("back")
		scene = menu:new()
	elseif key == "up" or key == "w" then
		self.cursor = self.cursor - 1
		if self.cursor < 1 then
			self.cursor = used
		end
	elseif key == "down" or key == "s" then
		self.cursor = self.cursor + 1
		if self.cursor > used then
			self.cursor = 1
		end
	elseif key == "return" or key == " " then
		self:input(self.cursor)
	elseif tonumber(key) then
		self:input(tonumber(key))
	elseif key == "f12" then
		local filename = fs:saveMap(self.grid)
		over:show("Map saved as "..filename)
	end
end

function game:mousereleased(x, y, button)
	self:input(self.cursor)
end

function game:input(num)
	if num == self.current or num > used or num < 1 then	-- check our input for invalid moves
		sfx:play("back")
	else
		sfx:play("confirm")
		self.current = num
		for y=1, 15 do		-- loop through the grid and try to seep out from each square
			for x=1, 15 do
				if self.grid[y][x] == 0 then
					self:seep(x, y)
				end
			end
		end
		self.moves = self.moves + 1
		if limit > 0 and self.moves >= limit then
			scene = fail:new()
		end
		self:winCheck()
	end
end

function game:seep(x, y)
	if x > 1 then	-- cannot dye left of first col
		self:dye(x-1, y)
	end
	if x < 15 then	-- cannot dye right of last col
		self:dye(x+1, y)
	end
	if y > 1 then	-- cannot dye above first row
		self:dye(x, y-1)
	end
	if y < 15 then	-- cannot dye below last row
		self:dye(x, y+1)
	end
end

function game:dye(x, y)
	if self.grid[y][x] == self.current then		-- check if we can dye this square
		self.grid[y][x] = 0
		self:seep(x, y)	-- check the neighboring squares, see if we can dye them too
	end
end

function game:winCheck()
	for x=1, 15 do
		for y=1, 15 do
			if self.grid[y][x] ~= 0 then
				return
			end
		end
	end
	scene = victory:new(self.moves)
end
