menu = class:new()

function menu:init()
	self.text = {}		-- holds what the user sees
	self.commands = {}	-- holds the lua each menu item executs
	self.place = "top"	-- where we are in the menu
	self.cursor = 1		-- where the cursor is on the menu
	self:process()		-- set up the top level
end

function menu:update(dt)
	local mX, mY = love.mouse.getPosition()
	if mX ~= mouseX or mY ~= mouseY then	-- only do when we move
		if mY > 240 and mY < (#self.text * 32) + 280 then	-- only do when we're in range
			self.cursor = math.min(math.max(math.floor((mY - 240) / 32), 1), #self.text)	-- find the cursor equivalent, and keep it in bounds
		end
		mouseX, mouseY = mX, mY
	end
end

function menu:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(logo, 160, 0, 0, 10, 10)
	for i=1, #self.text do
		love.graphics.print(self.text[i], 260, (i * 32) + 240)
	end
	love.graphics.setColor(colors[self.cursor])
	love.graphics.rectangle("fill", 220, (self.cursor * 32) + 248, 16, 16)
end

function menu:keypressed(key, unicode)
	if key == "escape" or key == "backspace" then
		if self.place == "top" then		-- we're as high as we can go, so quit
			love.event.push("q")
		else
			self.place = "top"		-- we must go shallower!
			self:process()
		end
		sfx:play("back")
	elseif key == "up" or key == "w" then
		self.cursor = self.cursor - 1
		if self.cursor < 1 then
			self.cursor = #self.text
		end
	elseif key == "down" or key == "s" then
		self.cursor = self.cursor + 1
		if self.cursor > #self.text then
			self.cursor = 1
		end
	elseif key == "return" or key == " " then
		loadstring(self.commands[self.cursor])()	-- execute the code that was stored in the commands table
		sfx:play("confirm")
	elseif tonumber(key) then
		local num = tonumber(key)
		if num > 0 and num <  #self.text then
			loadstring(self.commands[num])()	-- execute the code that was stored in the commands table
			sfx:play("confirm")
		end
	end
end

function menu:mousereleased(x, y, button)
	loadstring(self.commands[self.cursor])()	-- execute the code that was stored in the commands table
	sfx:play("confirm")
end

function menu:process(stay)
	stay = stay or false
	self.text = {}
	self.commands = {}
	local recur = [[
		scene:process()
	]]
	local back_btn = "top"
	if self.place == "top" then
		self:insert("New Game", [[
			scene.place = "select"
		]] .. recur)
		self:insert("Options", [[
			scene.place = "options"
		]] .. recur)
		self:insert("How To Play", [[
			over:show("Dye the whole map one color to win. Starting from the top left corner, change your dye color to claim adjacent squares of the same color. Try to use as few moves as possible to achieve a high score! \n\nControls: \nNumber Keys/Enter/Mouseclick: Change the color. \nUp/Down: Move the cursor.")
		]])
		self:insert("High Scores", [[
			over:show("High Scores: \n"..fs:loadScores())
		]])
		self:insert("About", [[
			over:show("Produced by Thomas Holden \ntsholden.com \n\nPowered by Love \nlove2d.org \n\nSECS by Bart Bes \ngithub.com/bartbes/")
		]])
		self:insert("Quit Game", [[
			love.event.push('q')
		]])
		back_btn = ""
	elseif self.place == "select" then
		self:insert("Random Map", [[
			scene = game:new()
		]])
		self:insert("Load Map", [[
			scene.place = "load"
		]] .. recur)
		self:insert("Limit Moves", [[
			scene.place = "limit"
		]] .. recur)
	elseif self.place == "options" then
		self:insert("Fullscreen", [[
			fullscreen = not fullscreen
			love.graphics.setMode(640, 480, fullscreen)
		]])
		self:insert("Mute SFX", [[
			muted = not muted
		]])
		self:insert("Increase Difficulty ("..used..")", [[
			if used < #colors then
				used = used + 1
			end
			scene:process(true)
		]])
		self:insert("Decrease Difficulty ("..used..")", [[
			if used > 2 then
				used = used - 1
			end
			scene:process(true)
		]])
	elseif self.place == "load" then
		local maps = saver:listMaps()
		if #maps > 0 then
			for k, v in pairs(maps) do 
				self:insert(v, [[
					saver:loadMap("]]..v..[[")
					scene = game:new()
				]])
			end
		else
			self:insert("No Saved Maps!", [[
				scene.place = "select"
			]] .. recur)
		end
		back_btn = "select"
	elseif self.place == "limit" then
		local limitstring = limit
		if limit == 0 then
			limitstring = "Infinite"
		end
		self:insert("Increase Limit ("..limitstring..")", [[
			limit = limit + 1
			scene:process(true)
		]])
		self:insert("Decrease Limit ("..limitstring..")", [[
			if limit > 0 then
				limit = limit - 1
			end
			scene:process(true)
		]])
		back_btn = "select"
	end
	if back_btn ~= "" then
		self:insert("Back", [[
			scene.place = "]]..back_btn..[["
		]] .. recur)
	end
	if not stay then
		self.cursor = 1
	end
end

function menu:insert(guistring, luastring)
	table.insert(self.text, guistring or " ")
	table.insert(self.commands, luastring or [[ ]])
end
