function love.load()
	require "lua/class"
	require "lua/sound"
	require "lua/saver"
	require "lua/overlay"
	require "lua/scene_menu"
	require "lua/scene_game"
	require "lua/scene_fail"
	require "lua/scene_victory"
	math.randomseed(os.time())	-- ensures truly random maps
	local wendy = love.graphics.newFont("media/wendy.ttf", 32)
	love.graphics.setFont(wendy)
	logo = love.graphics.newImage("media/logo.png")
	logo:setFilter("nearest", "nearest")	-- we do not want bluring
	love.graphics.setIcon(logo)
	mouseX, mouseY = love.mouse.getPosition()	-- used to detect change in mouse position
	fullscreen = false
	muted = false
	colors = {
		{210, 60, 30},	-- red
		{120, 150, 30},	-- green
		{60, 180, 210},	-- blue
		{240, 210, 30},	-- yellow
		{90, 90, 180},	-- purple
		{240, 120, 30},	-- orange
		{180, 120, 210},	-- pink
		{120, 120, 120}, 	-- dark gray
		{210, 210, 210} 	-- light grey
	}
	used = 5	-- # of colors used ingame, determines the difficulty
	limit = 0	-- limit number of moves (0 means infinite)
	sfx = sound:new()
	over = overlay:new()
	fs = saver:new()
	scene = menu:new()
end

function love.update(dt)
	over:update(dt)
	if not over.active then		-- we don't update the cursor if the overlay is active
		scene:update(dt)
	end
end

function love.draw()
	scene:draw()
	over:draw()
end

function love.keypressed(key, unicode)
	if over.active then
		over:hide()		-- kill the overlay w/out also triggering any other input
	else
		if string.sub(key, 1, 2) == "kp" then
			key = string.sub(key, 3)		-- cut numpad keys down to just numbers... ie, kp7 becomes just 7
		end
		scene:keypressed(key, unicode)
	end
end

function love.mousereleased(x, y, button)
	if over.active then
		over:hide()
	else
		scene:mousereleased(x, y, button)
	end
end
